package Agent ;

import java.lang.reflect.* ;
import java.io.* ;
import java.net.* ;

public abstract class Agent extends Thread implements java.io.Serializable {
  protected String AgentName ; // nome dell'agente
  protected boolean MessagesOn = false ;
  
  public Agent( String s ) {
    super( "Agent - " + s ) ;
    AgentName = getName() ;
  }

  public Agent() {
    super( "Agent" ) ;
    AgentName = getName() ;
  }

  final public String AgentName() {
    return AgentName ;
  }

  final public byte[] getClassBytes() {
    return getClassBytes( getClass().getName() ) ;
  }

  // versione corretta che funziona anche con Java 2
  public byte[] getClassBytes( String className ) {
    ClassLoader defaultLoader=this.getClass().getClassLoader();
    AgentClassLoader classLoader;

    if ( ( defaultLoader != null ) && 
         (defaultLoader instanceof AgentClassLoader) )  {
      //cio non sto utilizzando il class loader di default
    
      classLoader = (AgentClassLoader)(defaultLoader) ;
      byte[] ClassBytes = classLoader.getClassBytes( className ) ;
      if ( ClassBytes != null )
        return ClassBytes ;
    }
    //fine versione corretta

    try {
      byte result[] = ClassBytesLoader.loadClassBytes(className) ;
      return result ;
    } catch ( FileNotFoundException nof ) {
      nof.printStackTrace() ;
      return null ;
    } catch ( Exception e ) {
      e.printStackTrace() ;
    }
    return null ;
  }  

	protected void migrate( String host, int port ) throws AgentException {
	  Socket socket ;
	  try {
	    PrintMessage( "Trasferimento su " + host + ":" + port ) ;
	    socket = new Socket( host, port ) ;
      sendAgent( socket ) ;
	  } catch ( IOException e ) {
	    e.printStackTrace() ;
	  }
	}
	
	protected void sendAgent( Socket socket ) 
	    throws IOException, AgentException {
	      ObjectOutputStream objOStream =
	    new ObjectOutputStream( socket.getOutputStream() ) ;
	      ObjectInputStream objIStream =
	    new ObjectInputStream( socket.getInputStream() ) ;
      Boolean ok ;
      try {
        PrintMessage( "Preparazione dati per migrazione..." ) ;
        AgentPacket agpack = createAgentPacket() ;
        PrintMessage( "Migrazione..." ) ;
  	    objOStream.writeObject( agpack ) ;
  	    ok = (Boolean)(objIStream.readObject()) ;
  	    if ( ! ok.booleanValue() ) {
  	      throw new AgentException( "Migrazione fallita" ) ;
  	    }
        PrintMessage( "Migrazione eseguita." ) ;
	    } catch ( ClassNotFoundException e ) {
	      e.printStackTrace() ;
	    }
	}

  protected AgentPacket createAgentPacket() {
    return new AgentPacket( this, getClassBytes() ) ;
  }
		  
	final public void run() {
		try {
		    execute() ;
		} catch ( AgentException e ) {
		    System.err.println( "Eccezione non gestita " + e ) ;
		    e.printStackTrace() ;
		}
	}

	// da definire nelle classi derivate
	abstract protected void execute() throws AgentException ;
	// chiamato all'arrivo su un sito remoto
	abstract public void onArrival() throws AgentException ;

	synchronized public void Print( String s ) {
		System.out.println( s ) ;
	}

	public void pause( int TimeToPause ) {
		try {
			Thread.sleep( TimeToPause ) ;
		} catch ( InterruptedException e ) {
			System.err.println( e ) ;
		}
	}

  synchronized public void setMessages( boolean on ) { MessagesOn = on ; }

  synchronized protected void PrintMessage( String s ) {
    if ( MessagesOn )
      System.out.println( s ) ;
  }
}

