package Agent ;

import java.io.* ;
import java.util.Hashtable ;
import java.util.StringTokenizer ;

public class AgentClassLoader extends ClassLoader {
  private Hashtable classData = new Hashtable();
  protected boolean MessagesOn = false ;

  public AgentClassLoader() {
  }

  synchronized public void addClassBytes( String className, byte classBytes[] ) {
    if( classData.get( className ) == null && classBytes != null )
      classData.put( className, classBytes ) ;
  }

  public byte[] getClassBytes(String className) {
    return (byte[]) classData.get( className ) ;
  }

  synchronized public void setMessages( boolean on ) { MessagesOn = on ; }

  public Class loadClass(String className) throws ClassNotFoundException {
    return (loadClass(className, true));
  }

  protected synchronized Class loadClass(String className, boolean resolveIt)
    throws ClassNotFoundException {
      return loadClass( className, resolveIt, false ) ;
  }

  protected synchronized Class loadClass(String className, boolean resolveIt,
        boolean force )
    throws ClassNotFoundException {
      Class result;
      byte  classData[];

      PrintMessage("--- caricamento : " + className );

      /* proviamo prima con la cache */
      result = findLoadedClass(className);
      if (result != null) {
	      PrintMessage("        --- recuperata dalla cache." );
        return result;
      }

      if ( ! force ) {
        try {
          result = super.findSystemClass(className);
          PrintMessage("        --- caricata dal file system locale");
          return result;
        } catch (ClassNotFoundException e) {
          PrintMessage("        --- non e' una classe di sistema");
        }
      }

      if ( className.startsWith( "java." ) ) {
        // pericolo !!!
        throw new SecurityException( className ) ;
      }

      /* proviamo a cercare nella nostra tabella */
      classData = getClassBytes(className);
      if (classData != null) {
        PrintMessage("        --- recupero dei bytes di " + className );
      } else {
        ClassNotFoundException e = new ClassNotFoundException( className ) ;
        e.printStackTrace() ;
        throw e ;
      }

      /* parsing */
      result = defineClass(className, classData, 0, classData.length);
      if (result == null) {
        throw new ClassFormatError();
      }

      if (resolveIt) {
        resolveClass(result);
      }

      // PrintMessage("            --- nuova classe caricata : " + className );
      System.out.println("            --- nuova classe caricata : " + className );
      return result;
  }

  public synchronized Class forceLoadClass(String className)
    throws ClassNotFoundException {
      return loadClass( className, true, true ) ;
  }

  synchronized protected void PrintMessage( String s ) {
    if ( MessagesOn )
      System.out.println( s ) ;
  }
}
